package com.jiuqi.util.chinese;

import java.io.Serializable;

class ChineseDigitUnit implements Cloneable, Serializable
{
    private static final char[] SMALL_UNIT;
    private static final char[] BIG_UNIT;
    private static final char[] LARGE_UNIT;
    private static String bufferedUnit;
    private static String bufferedPower;
    private String power;
    
    static {
        SMALL_UNIT = new char[] { '十', '百', '千' };
        BIG_UNIT = new char[] { '拾', '佰', '仟' };
        LARGE_UNIT = new char[] { '万', '亿' };
        ChineseDigitUnit.bufferedUnit = null;
        ChineseDigitUnit.bufferedPower = null;
    }
    
    public ChineseDigitUnit() {
        this.power = "1";
    }
    
    public ChineseDigitUnit(long power) {
        if (power < 0L) {
            power = -power;
        }
        long value;
        for (value = 1L; power >= 10L; power /= 10L, value *= 10L) {}
        this.power = String.valueOf(value);
    }
    
    public ChineseDigitUnit(final String unit) {
        if (unit != null && unit.equals("")) {
            final boolean digitString = isDigitString(unit);
            if (!digitString) {
                try {
                    this.power = valueOf(unit);
                    return;
                }
                catch (final NotValidChineseDigitUnitException ex) {
                    throw new IllegalArgumentException("单位\"" + unit + "\"既不是一个有效的阿拉伯数字，也不是有效的中文数字单位！");
                }
                throw new IllegalArgumentException("字符串为null或者空字符串！");
            }
            final StringBuffer sb = new StringBuffer("1");
            for (int i = 0; i < unit.length() - 1; ++i) {
                sb.append("0");
            }
            this.power = sb.toString();
            return;
        }
        throw new IllegalArgumentException("字符串为null或者空字符串！");
    }
    
    public long longValue() {
        return Long.parseLong(this.power);
    }
    
    public String chineseValue(final boolean useBigChar) {
        return toChineseUnit(this.power, useBigChar);
    }
    
    public static String valueOf(final String value) throws NotValidChineseDigitUnitException {
        if (value.equals(ChineseDigitUnit.bufferedUnit)) {
            return ChineseDigitUnit.bufferedPower;
        }
        long result = 0L;
        for (int i = 0; i < value.length(); ++i) {
            final char thisUnit = value.charAt(i);
            int position = -1;
            for (int j = 0; j < ChineseDigitUnit.SMALL_UNIT.length; ++j) {
                if (thisUnit == ChineseDigitUnit.SMALL_UNIT[j]) {
                    position = j;
                    break;
                }
            }
            if (position != -1) {
                result += position + 1;
                position = -1;
            }
            else {
                for (int j = 0; j < ChineseDigitUnit.BIG_UNIT.length; ++j) {
                    if (thisUnit == ChineseDigitUnit.BIG_UNIT[j]) {
                        position = j;
                        break;
                    }
                }
                if (position != -1) {
                    result += position + 1;
                    position = -1;
                }
                else {
                    for (int j = 0; j < ChineseDigitUnit.LARGE_UNIT.length; ++j) {
                        if (thisUnit == ChineseDigitUnit.LARGE_UNIT[j]) {
                            position = j;
                            break;
                        }
                    }
                    if (position == -1) {
                        throw new NotValidChineseDigitUnitException("字符" + thisUnit + "不是一个合法的中文数字单位！");
                    }
                    result += 4 * (position + 1);
                    position = -1;
                }
            }
        }
        final StringBuffer sb = new StringBuffer("1");
        for (int k = 0; k < result; ++k) {
            sb.append("0");
        }
        ChineseDigitUnit.bufferedUnit = value;
        ChineseDigitUnit.bufferedPower = sb.toString();
        return sb.toString();
    }
    
    public static String toChineseUnit(final String power, final boolean useBigChar) {
        if (isDigitString(power)) {
            int pow = power.length();
            final StringBuffer sb = new StringBuffer();
            while (pow > 8) {
                sb.insert(0, ChineseDigitUnit.LARGE_UNIT[1]);
                pow -= 8;
            }
            while (pow > 4) {
                sb.insert(0, ChineseDigitUnit.LARGE_UNIT[0]);
                pow -= 4;
            }
            if (pow > 1) {
                if (!useBigChar) {
                    sb.insert(0, ChineseDigitUnit.SMALL_UNIT[pow - 2]);
                }
                else {
                    sb.insert(0, ChineseDigitUnit.BIG_UNIT[pow - 2]);
                }
            }
            return sb.toString();
        }
        throw new IllegalArgumentException("字符串\"" + power + "\"不是一个阿拉伯数字字符串！");
    }
    
    static boolean isDigitString(final String value) {
        for (int i = 0; i < value.length(); ++i) {
            final char c = value.charAt(i);
            if (!Character.isDigit(c)) {
                return false;
            }
        }
        return true;
    }
    
    public static boolean isValidChineseDigitUnit(final String value) {
        try {
            valueOf(value);
            return true;
        }
        catch (final NotValidChineseDigitUnitException ex) {
            return false;
        }
    }
    
    public static boolean isLargeUnit(final char value) {
        for (int i = 0; i < ChineseDigitUnit.LARGE_UNIT.length; ++i) {
            if (value == ChineseDigitUnit.LARGE_UNIT[i]) {
                return true;
            }
        }
        return false;
    }
    
    public Object clone() {
        return new ChineseDigitUnit(this.power);
    }
    
    @Override
    public boolean equals(final Object obj) {
        if (this == obj) {
            return true;
        }
        if (!(obj instanceof ChineseDigitUnit)) {
            return false;
        }
        final ChineseDigitUnit unit = (ChineseDigitUnit)obj;
        return this.power.equals(unit.power);
    }
    
    @Override
    public int hashCode() {
        int value = super.hashCode();
        if (this.power != null) {
            value ^= this.power.hashCode();
        }
        return value;
    }
    
    @Override
    public String toString() {
        return this.chineseValue(false);
    }
}
